# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SourceRateLimit(UniversalBaseModel):
    """
    Complete source rate limit schema.
    """

    source_short_name: str = pydantic.Field()
    """
    Source identifier (e.g., 'google_drive', 'notion')
    """

    limit: int = pydantic.Field()
    """
    Maximum requests allowed per window
    """

    window_seconds: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time window in seconds (60=per minute, 86400=per day, etc.)
    """

    id: str
    organization_id: str
    created_at: dt.datetime
    modified_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audit_action_type import AuditActionType


class CatalogAuditLog(UniversalBaseModel):
    """
    Schema for audit log responses.
    """

    action_type: AuditActionType
    action_description: str
    user_email: typing.Optional[str] = None
    user_name: typing.Optional[str] = None
    details: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    id: str
    catalog_item_id: str
    sync_job_id: typing.Optional[str] = None
    source_name: typing.Optional[str] = None
    ip_address: typing.Optional[str] = None
    user_agent: typing.Optional[str] = None
    processing_time_ms: typing.Optional[int] = None
    error_message: typing.Optional[str] = None
    created_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .billing_period_status import BillingPeriodStatus
from .billing_plan import BillingPlan
from .billing_transition import BillingTransition


class BillingPeriod(UniversalBaseModel):
    """
    Complete billing period representation.
    """

    organization_id: str = pydantic.Field()
    """
    Organization this period belongs to
    """

    period_start: dt.datetime = pydantic.Field()
    """
    Period start (inclusive)
    """

    period_end: dt.datetime = pydantic.Field()
    """
    Period end (exclusive)
    """

    plan: BillingPlan = pydantic.Field()
    """
    Plan for this period
    """

    status: BillingPeriodStatus = pydantic.Field()
    """
    Period status
    """

    created_from: BillingTransition = pydantic.Field()
    """
    How this period was created
    """

    id: str
    stripe_subscription_id: typing.Optional[str] = None
    stripe_invoice_id: typing.Optional[str] = None
    amount_cents: typing.Optional[int] = None
    currency: typing.Optional[str] = None
    paid_at: typing.Optional[dt.datetime] = None
    previous_period_id: typing.Optional[str] = None
    created_at: dt.datetime
    modified_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .catalog_status import CatalogStatus
from .sensitivity_level import SensitivityLevel


class CatalogItemGovernanceUpdate(UniversalBaseModel):
    """
    Schema for updating governance-related fields only.

    Restricted to fields that users should be able to modify manually.
    """

    status: typing.Optional[CatalogStatus] = None
    description: typing.Optional[str] = None
    tags: typing.Optional[typing.List[str]] = None
    sensitivity_level: typing.Optional[SensitivityLevel] = None
    reason: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .collection_status import CollectionStatus


class Collection(UniversalBaseModel):
    """
    Complete collection representation returned by the API.

    This schema includes all collection metadata plus computed status information
    based on the health and state of associated source connections.
    """

    name: str = pydantic.Field()
    """
    Human-readable display name for the collection.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the collection.
    """

    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Color for the collection.
    """

    icon: typing.Optional[str] = pydantic.Field(default=None)
    """
    Icon for the collection.
    """

    readable_id: str = pydantic.Field()
    """
    URL-safe unique identifier used in API endpoints. This becomes non-optional once the collection is created.
    """

    id: str = pydantic.Field()
    """
    Unique system identifier for the collection. This UUID is generated automatically and used for internal references.
    """

    vector_size: int = pydantic.Field()
    """
    Vector dimensions used by this collection. Determines which embedding model is used: 3072 (text-embedding-3-large), 1536 (text-embedding-3-small), or 384 (MiniLM-L6-v2).
    """

    embedding_model_name: str = pydantic.Field()
    """
    Name of the embedding model used for this collection (e.g., 'text-embedding-3-large', 'text-embedding-3-small'). This ensures queries use the same model as the indexed data.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the collection was created (ISO 8601 format).
    """

    modified_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the collection was last modified (ISO 8601 format).
    """

    organization_id: str = pydantic.Field()
    """
    Identifier of the organization that owns this collection. Collections are isolated per organization.
    """

    created_by_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address of the user who created this collection.
    """

    modified_by_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address of the user who last modified this collection.
    """

    status: typing.Optional[CollectionStatus] = pydantic.Field(default=None)
    """
    Current operational status of the collection:<br/>• **NEEDS_SOURCE**: Collection has no authenticated connections, or connections exist but haven't synced yet<br/>• **ACTIVE**: At least one connection has completed a sync or is currently syncing<br/>• **ERROR**: All connections have failed their last sync
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .agent_api_key_summary import AgentApiKeySummary


class AgentApiKeyCollectionStats(UniversalBaseModel):
    """
    Collection-level statistics for agent API keys.
    """

    total_keys: typing.Optional[int] = None
    active_keys: typing.Optional[int] = None
    revoked_keys: typing.Optional[int] = None
    suspended_keys: typing.Optional[int] = None
    expired_keys: typing.Optional[int] = None
    keys_needing_renewal: typing.Optional[int] = None
    total_requests_last_24_h: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="total_requests_last_24h")
    ] = None
    unique_clusters_accessed: typing.Optional[int] = None
    most_active_key: typing.Optional[AgentApiKeySummary] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .catalog_item_summary import CatalogItemSummary


class CatalogTreeNode(UniversalBaseModel):
    """
    Node in a catalog tree view.
    """

    catalog_item: CatalogItemSummary
    children: typing.Optional[typing.List["CatalogTreeNode"]] = None
    is_expanded: typing.Optional[bool] = None
    selection_state: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(CatalogTreeNode)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .catalog_audit_log_summary import CatalogAuditLogSummary


class RagGovernanceReport(UniversalBaseModel):
    """
    Report on RAG inclusion/exclusion changes.
    """

    total_inclusion_changes: int
    total_exclusion_changes: int
    recent_changes: typing.List[CatalogAuditLogSummary]
    most_active_users: typing.List[str]
    bulk_operations: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .response_type import ResponseType
from .search_status import SearchStatus


class LegacySearchResponse(UniversalBaseModel):
    """
    Legacy search response schema for backwards compatibility.
    """

    results: typing.List[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field()
    """
    Array of search result objects
    """

    response_type: ResponseType = pydantic.Field()
    """
    Indicates whether results are raw search matches or AI-generated completions
    """

    completion: typing.Optional[str] = pydantic.Field(default=None)
    """
    AI-generated natural language answer when response_type is 'completion'
    """

    status: SearchStatus = pydantic.Field()
    """
    Status of the search operation
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .collection_role import CollectionRole


class CollectionMember(UniversalBaseModel):
    """
    Schema representing a collection member.
    """

    user_id: str = pydantic.Field()
    """
    Unique identifier of the user
    """

    user_email: str = pydantic.Field()
    """
    Email address of the user
    """

    role: CollectionRole = pydantic.Field()
    """
    Role of the user in this collection
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the user was added to the collection
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .legacy_search_request_search_method import LegacySearchRequestSearchMethod
from .query_expansion_strategy import QueryExpansionStrategy
from .response_type import ResponseType


class LegacySearchRequest(UniversalBaseModel):
    """
    Legacy search request schema for backwards compatibility.
    """

    query: str = pydantic.Field()
    """
    The search query text (max 2048 tokens)
    """

    filter: typing.Optional["Filter"] = pydantic.Field(default=None)
    """
    Qdrant native filter for metadata-based filtering
    """

    offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of results to skip
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of results
    """

    score_threshold: typing.Optional[float] = pydantic.Field(default=None)
    """
    Minimum similarity score threshold (deprecated, will be ignored)
    """

    response_type: typing.Optional[ResponseType] = pydantic.Field(default=None)
    """
    Type of response - 'raw' or 'completion'
    """

    search_method: typing.Optional[LegacySearchRequestSearchMethod] = pydantic.Field(default=None)
    """
    Search method to use
    """

    recency_bias: typing.Optional[float] = pydantic.Field(default=None)
    """
    How much document age affects the similarity score (0..1)
    """

    expansion_strategy: typing.Optional[QueryExpansionStrategy] = pydantic.Field(default=None)
    """
    Query expansion strategy
    """

    enable_reranking: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable LLM-based reranking to improve result relevance
    """

    enable_query_interpretation: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable automatic filter extraction from natural language query
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .filter import Filter  # noqa: E402, F401, I001
from .min_should import MinShould  # noqa: E402, F401, I001
from .nested import Nested  # noqa: E402, F401, I001
from .nested_condition import NestedCondition  # noqa: E402, F401, I001

update_forward_refs(LegacySearchRequest)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UsageSnapshot(UniversalBaseModel):
    """
    Point-in-time usage data with limits.
    """

    entities: int = pydantic.Field()
    """
    Total entities processed
    """

    queries: int = pydantic.Field()
    """
    Search queries executed
    """

    source_connections: int = pydantic.Field()
    """
    Source connections configured
    """

    team_members: int = pydantic.Field()
    """
    Team members in the organization
    """

    max_entities: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum entities allowed
    """

    max_queries: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum queries allowed
    """

    max_source_connections: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum source connections allowed
    """

    max_team_members: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum team members allowed
    """

    timestamp: dt.datetime = pydantic.Field()
    """
    When this snapshot was taken
    """

    billing_period_id: str = pydantic.Field()
    """
    Associated billing period
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .retrieval_strategy import RetrievalStrategy


class SearchRequest(UniversalBaseModel):
    """
    Search request schema.
    """

    query: str = pydantic.Field()
    """
    The search query text
    """

    context_cluster_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Optional list of context cluster IDs to restrict results to
    """

    retrieval_strategy: typing.Optional[RetrievalStrategy] = pydantic.Field(default=None)
    """
    The retrieval strategy to use
    """

    filter: typing.Optional["Filter"] = pydantic.Field(default=None)
    """
    Qdrant native filter for metadata-based filtering
    """

    offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of results to skip
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of results to return
    """

    temporal_relevance: typing.Optional[float] = pydantic.Field(default=None)
    """
    Weight recent content higher than older content; 0 = no recency effect, 1 = only recent items matter
    """

    expand_query: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Generate a few query variations to improve recall
    """

    interpret_filters: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Extract structured filters from natural-language query
    """

    rerank: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Reorder the top candidate results for improved relevance. Max number of results that can be reranked is capped to around 1000.
    """

    generate_answer: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Generate a natural-language answer to the query
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .filter import Filter  # noqa: E402, F401, I001
from .min_should import MinShould  # noqa: E402, F401, I001
from .nested import Nested  # noqa: E402, F401, I001
from .nested_condition import NestedCondition  # noqa: E402, F401, I001

update_forward_refs(SearchRequest)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .feature_flag import FeatureFlag
from .organization_billing import OrganizationBilling


class OrganizationWithRole(UniversalBaseModel):
    """
    Organization schema with user's role information.
    """

    id: str
    name: str
    description: typing.Optional[str] = None
    created_at: dt.datetime
    modified_at: dt.datetime
    role: str
    is_primary: bool
    auth_0_org_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="auth0_org_id")] = None
    org_type: typing.Optional[str] = None
    org_metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    enabled_features: typing.Optional[typing.List[FeatureFlag]] = pydantic.Field(default=None)
    """
    List of enabled feature flags for this organization
    """

    billing: typing.Optional[OrganizationBilling] = pydantic.Field(default=None)
    """
    Complete billing information including current period
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs


class Filter(UniversalBaseModel):
    should: typing.Optional["Should"] = pydantic.Field(default=None)
    """
    At least one of those conditions should match
    """

    min_should: typing.Optional["MinShould"] = pydantic.Field(default=None)
    """
    At least minimum amount of given conditions should match
    """

    must: typing.Optional["Must"] = pydantic.Field(default=None)
    """
    All conditions must match
    """

    must_not: typing.Optional["FilterMustNot"] = pydantic.Field(default=None)
    """
    All conditions must NOT match
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .min_should import MinShould  # noqa: E402, F401, I001
from .nested import Nested  # noqa: E402, F401, I001
from .nested_condition import NestedCondition  # noqa: E402, F401, I001
from .filter_should_item import FilterShouldItem  # noqa: E402, F401, I001
from .must import Must  # noqa: E402, F401, I001
from .filter_must_item import FilterMustItem  # noqa: E402, F401, I001
from .should import Should  # noqa: E402, F401, I001
from .filter_must_not_item import FilterMustNotItem  # noqa: E402, F401, I001
from .min_should_conditions_item import MinShouldConditionsItem  # noqa: E402, F401, I001
from .filter_must_not import FilterMustNot  # noqa: E402, F401, I001

update_forward_refs(Filter)



# This file was auto-generated by Fern from our API Definition.

import typing

BillingTransition = typing.Union[
    typing.Literal["initial_signup", "renewal", "upgrade", "downgrade", "reactivation", "trial_conversion"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .billing_period_usage import BillingPeriodUsage
from .usage_trend import UsageTrend


class UsageDashboard(UniversalBaseModel):
    """
    Complete dashboard data.
    """

    current_period: BillingPeriodUsage = pydantic.Field()
    """
    Current billing period usage
    """

    previous_periods: typing.Optional[typing.List[BillingPeriodUsage]] = pydantic.Field(default=None)
    """
    Historical billing periods
    """

    total_entities_all_time: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total entities processed all time
    """

    total_queries_all_time: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total queries executed all time
    """

    average_daily_entities: typing.Optional[int] = pydantic.Field(default=None)
    """
    Average entities per day (current period)
    """

    average_daily_queries: typing.Optional[int] = pydantic.Field(default=None)
    """
    Average queries per day (current period)
    """

    trends: typing.Optional[typing.List[UsageTrend]] = pydantic.Field(default=None)
    """
    Usage trends compared to previous period
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SourceRateLimitResponse(UniversalBaseModel):
    """
    Response schema with source metadata merged for UI display.
    """

    source_short_name: str
    rate_limit_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    'org' (organization-wide), 'connection' (per-user), or None (not supported)
    """

    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Configured limit, None if not set
    """

    window_seconds: typing.Optional[int] = pydantic.Field(default=None)
    """
    Configured window, None if not set
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    DB record ID, None if not configured
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ApiKey(UniversalBaseModel):
    """
    Schema for API keys returned to clients - includes decrypted key.
    """

    id: str
    organization_id: str
    created_at: dt.datetime
    modified_at: dt.datetime
    last_used_date: typing.Optional[dt.datetime] = None
    expiration_date: dt.datetime
    created_by_email: typing.Optional[str] = None
    modified_by_email: typing.Optional[str] = None
    decrypted_key: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class OAuthBrowserAuthentication(UniversalBaseModel):
    """
    OAuth authentication via browser flow.

    Supports both OAuth2 and OAuth1 BYOC (Bring Your Own Client):
    - OAuth2 BYOC: Provide client_id + client_secret
    - OAuth1 BYOC: Provide consumer_key + consumer_secret
    """

    redirect_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth redirect URI
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth2 client ID (for custom apps)
    """

    client_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth2 client secret (for custom apps)
    """

    consumer_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth1 consumer key (for custom apps)
    """

    consumer_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth1 consumer secret (for custom apps)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .single_action_check_response_reason import SingleActionCheckResponseReason


class SingleActionCheckResponse(UniversalBaseModel):
    """
    Response schema for single action permission check.
    """

    allowed: bool = pydantic.Field()
    """
    Whether the action is allowed for the organization
    """

    action: str = pydantic.Field()
    """
    The action type that was checked
    """

    reason: typing.Optional[SingleActionCheckResponseReason] = pydantic.Field(default=None)
    """
    Reason why the action is not allowed (if applicable)
    """

    details: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Additional details about why the action is not allowed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .sync_job_status import SyncJobStatus


class SyncJob(UniversalBaseModel):
    """
    Schema for SyncJob.
    """

    sync_id: str
    status: typing.Optional[SyncJobStatus] = None
    scheduled: typing.Optional[bool] = None
    entities_inserted: typing.Optional[int] = None
    entities_updated: typing.Optional[int] = None
    entities_deleted: typing.Optional[int] = None
    entities_kept: typing.Optional[int] = None
    entities_skipped: typing.Optional[int] = None
    entities_encountered: typing.Optional[typing.Dict[str, typing.Optional[int]]] = None
    started_at: typing.Optional[dt.datetime] = None
    completed_at: typing.Optional[dt.datetime] = None
    failed_at: typing.Optional[dt.datetime] = None
    error: typing.Optional[str] = None
    access_token: typing.Optional[str] = None
    id: str
    organization_id: str
    created_by_email: typing.Optional[str] = None
    modified_by_email: typing.Optional[str] = None
    created_at: typing.Optional[dt.datetime] = None
    modified_at: typing.Optional[dt.datetime] = None
    sync_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the sync, populated from join query
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SearchResponse(UniversalBaseModel):
    """
    Comprehensive search response containing results and metadata.
    """

    results: typing.List[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field()
    """
    Array of search result objects containing the found documents, records, or data entities.
    """

    completion: typing.Optional[str] = pydantic.Field(default=None)
    """
    This provides natural language answers to your query based on the content found across your connected data sources when generate_answer is true.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .geo_bounding_box import GeoBoundingBox
from .geo_polygon import GeoPolygon
from .geo_radius import GeoRadius
from .match import Match
from .range import Range
from .values_count import ValuesCount


class FieldCondition(UniversalBaseModel):
    """
    All possible payload filtering conditions
    """

    key: str = pydantic.Field()
    """
    Payload key
    """

    match: typing.Optional[Match] = pydantic.Field(default=None)
    """
    Check if point has field with a given value
    """

    range: typing.Optional[Range] = pydantic.Field(default=None)
    """
    Check if points value lies in a given range
    """

    geo_bounding_box: typing.Optional[GeoBoundingBox] = pydantic.Field(default=None)
    """
    Check if points geolocation lies in a given area
    """

    geo_radius: typing.Optional[GeoRadius] = pydantic.Field(default=None)
    """
    Check if geo point is within a given radius
    """

    geo_polygon: typing.Optional[GeoPolygon] = pydantic.Field(default=None)
    """
    Check if geo point is within a given polygon
    """

    values_count: typing.Optional[ValuesCount] = pydantic.Field(default=None)
    """
    Check number of values of the field
    """

    is_empty: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Check that the field is empty, alternative syntax for `is_empty: 'field_name'`
    """

    is_null: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Check that the field is null, alternative syntax for `is_null: 'field_name'`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_activity_summary import UserActivitySummary


class ComplianceReport(UniversalBaseModel):
    """
    Compliance report for audit purposes.
    """

    report_period: str
    total_catalog_items: int
    items_reviewed: int
    governance_actions_taken: int
    user_activity_breakdown: typing.List[UserActivitySummary]
    automated_vs_manual_ratio: typing.Dict[str, int]
    compliance_score: float
    recommendations: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Range(UniversalBaseModel):
    """
    Range filter request
    """

    lt: typing.Optional[float] = pydantic.Field(default=None)
    """
    point.key &lt; range.lt
    """

    gt: typing.Optional[float] = pydantic.Field(default=None)
    """
    point.key &gt; range.gt
    """

    gte: typing.Optional[float] = pydantic.Field(default=None)
    """
    point.key &gt;= range.gte
    """

    lte: typing.Optional[float] = pydantic.Field(default=None)
    """
    point.key &lt;= range.lte
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .authentication_method import AuthenticationMethod


class AuthenticationDetails(UniversalBaseModel):
    """
    Authentication information.
    """

    method: AuthenticationMethod
    authenticated: bool
    authenticated_at: typing.Optional[dt.datetime] = None
    expires_at: typing.Optional[dt.datetime] = None
    auth_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    For pending OAuth flows
    """

    auth_url_expires: typing.Optional[dt.datetime] = None
    redirect_url: typing.Optional[str] = None
    provider_readable_id: typing.Optional[str] = None
    provider_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.schedule_config import ScheduleConfig
from ..types.source_connection import SourceConnection
from ..types.source_connection_job import SourceConnectionJob
from ..types.source_connection_list_item import SourceConnectionListItem
from .raw_client import AsyncRawSourceConnectionsClient, RawSourceConnectionsClient
from .types.authentication import Authentication

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SourceConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSourceConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSourceConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSourceConnectionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        collection: typing.Optional[str] = None,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SourceConnectionListItem]:
        """
        List source connections with minimal fields for performance.

        Parameters
        ----------
        collection : typing.Optional[str]
            Filter by collection readable ID

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionListItem]
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.list(
            collection="collection",
            skip=1,
            limit=1,
        )
        """
        _response = self._raw_client.list(
            collection=collection, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    def create(
        self,
        *,
        short_name: str,
        readable_collection_id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        schedule: typing.Optional[ScheduleConfig] = OMIT,
        sync_immediately: typing.Optional[bool] = OMIT,
        authentication: typing.Optional[Authentication] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Create a new source connection.

        The authentication configuration determines the flow:
        - DirectAuthentication: Immediate creation with provided credentials
        - OAuthBrowserAuthentication: Returns shell with authentication URL
        - OAuthTokenAuthentication: Immediate creation with provided token
        - AuthProviderAuthentication: Using external auth provider

        BYOC (Bring Your Own Client) is detected when client_id and client_secret
        are provided in OAuthBrowserAuthentication.

        sync_immediately defaults:
        - True for: direct, oauth_token, auth_provider
        - False for: oauth_browser, oauth_byoc (these sync after authentication)

        Parameters
        ----------
        short_name : str
            Source identifier (e.g., 'slack', 'github')

        readable_collection_id : str
            Collection readable ID

        name : typing.Optional[str]
            Connection name (defaults to '{Source Name} Connection')

        description : typing.Optional[str]
            Connection description

        config : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Source-specific configuration

        schedule : typing.Optional[ScheduleConfig]

        sync_immediately : typing.Optional[bool]
            Run initial sync after creation. Defaults to True for direct/token/auth_provider, False for OAuth browser/BYOC flows (which sync after authentication)

        authentication : typing.Optional[Authentication]
            Authentication config (defaults to OAuth browser flow for OAuth sources)

        redirect_url : typing.Optional[str]
            URL to redirect to after OAuth flow completes (only used for OAuth flows)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.create(
            short_name="short_name",
            readable_collection_id="readable_collection_id",
        )
        """
        _response = self._raw_client.create(
            short_name=short_name,
            readable_collection_id=readable_collection_id,
            name=name,
            description=description,
            config=config,
            schedule=schedule,
            sync_immediately=sync_immediately,
            authentication=authentication,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, source_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnection:
        """
        Get a source connection with optional depth expansion.

        Parameters
        ----------
        source_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.get(
            source_connection_id="source_connection_id",
        )
        """
        _response = self._raw_client.get(source_connection_id, request_options=request_options)
        return _response.data

    def delete(
        self, source_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnection:
        """
        Delete a source connection and all related data.

        Parameters
        ----------
        source_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.delete(
            source_connection_id="source_connection_id",
        )
        """
        _response = self._raw_client.delete(source_connection_id, request_options=request_options)
        return _response.data

    def update(
        self,
        source_connection_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        schedule: typing.Optional[ScheduleConfig] = OMIT,
        authentication: typing.Optional[Authentication] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Update a source connection.

        Updateable fields:
        - name, description
        - config_fields
        - cron_schedule
        - auth_fields (direct auth only)

        Parameters
        ----------
        source_connection_id : str

        name : typing.Optional[str]

        description : typing.Optional[str]

        config : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Source-specific configuration

        schedule : typing.Optional[ScheduleConfig]

        authentication : typing.Optional[Authentication]
            Authentication config (defaults to OAuth browser flow for OAuth sources)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.update(
            source_connection_id="source_connection_id",
        )
        """
        _response = self._raw_client.update(
            source_connection_id,
            name=name,
            description=description,
            config=config,
            schedule=schedule,
            authentication=authentication,
            request_options=request_options,
        )
        return _response.data

    def run(
        self,
        source_connection_id: str,
        *,
        force_full_sync: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnectionJob:
        """
        Trigger a sync run for a source connection.

        Runs are always executed through Temporal workflow engine.

        Args:
            db: Database session
            source_connection_id: ID of the source connection to run
            ctx: API context with organization and user information
            guard_rail: Guard rail service for usage limits
            force_full_sync: If True, forces a full sync with orphaned entity cleanup
                            for continuous syncs. Raises 400 error if used on
                            non-continuous syncs (which are always full syncs).

        Parameters
        ----------
        source_connection_id : str

        force_full_sync : typing.Optional[bool]
            Force a full sync ignoring cursor data instead of waiting for the daily cleanup schedule. Only allowed for continuous syncs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnectionJob
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.run(
            source_connection_id="source_connection_id",
            force_full_sync=True,
        )
        """
        _response = self._raw_client.run(
            source_connection_id, force_full_sync=force_full_sync, request_options=request_options
        )
        return _response.data

    def get_source_connection_jobs(
        self,
        source_connection_id: str,
        *,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SourceConnectionJob]:
        """
        Get sync jobs for a source connection.

        Parameters
        ----------
        source_connection_id : str

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionJob]
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.get_source_connection_jobs(
            source_connection_id="source_connection_id",
            limit=1,
        )
        """
        _response = self._raw_client.get_source_connection_jobs(
            source_connection_id, limit=limit, request_options=request_options
        )
        return _response.data

    def cancel_job(
        self, source_connection_id: str, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnectionJob:
        """
        Cancel a running sync job for a source connection.

        This endpoint requests cancellation and marks the job as CANCELLING.
        The workflow updates the final status to CANCELLED when it processes
        the cancellation request.

        Parameters
        ----------
        source_connection_id : str

        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnectionJob
            Successful Response

        Examples
        --------
        from graffo import GraffoSDK

        client = GraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )
        client.source_connections.cancel_job(
            source_connection_id="source_connection_id",
            job_id="job_id",
        )
        """
        _response = self._raw_client.cancel_job(source_connection_id, job_id, request_options=request_options)
        return _response.data


class AsyncSourceConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSourceConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSourceConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSourceConnectionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        collection: typing.Optional[str] = None,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SourceConnectionListItem]:
        """
        List source connections with minimal fields for performance.

        Parameters
        ----------
        collection : typing.Optional[str]
            Filter by collection readable ID

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionListItem]
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.list(
                collection="collection",
                skip=1,
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            collection=collection, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    async def create(
        self,
        *,
        short_name: str,
        readable_collection_id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        schedule: typing.Optional[ScheduleConfig] = OMIT,
        sync_immediately: typing.Optional[bool] = OMIT,
        authentication: typing.Optional[Authentication] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Create a new source connection.

        The authentication configuration determines the flow:
        - DirectAuthentication: Immediate creation with provided credentials
        - OAuthBrowserAuthentication: Returns shell with authentication URL
        - OAuthTokenAuthentication: Immediate creation with provided token
        - AuthProviderAuthentication: Using external auth provider

        BYOC (Bring Your Own Client) is detected when client_id and client_secret
        are provided in OAuthBrowserAuthentication.

        sync_immediately defaults:
        - True for: direct, oauth_token, auth_provider
        - False for: oauth_browser, oauth_byoc (these sync after authentication)

        Parameters
        ----------
        short_name : str
            Source identifier (e.g., 'slack', 'github')

        readable_collection_id : str
            Collection readable ID

        name : typing.Optional[str]
            Connection name (defaults to '{Source Name} Connection')

        description : typing.Optional[str]
            Connection description

        config : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Source-specific configuration

        schedule : typing.Optional[ScheduleConfig]

        sync_immediately : typing.Optional[bool]
            Run initial sync after creation. Defaults to True for direct/token/auth_provider, False for OAuth browser/BYOC flows (which sync after authentication)

        authentication : typing.Optional[Authentication]
            Authentication config (defaults to OAuth browser flow for OAuth sources)

        redirect_url : typing.Optional[str]
            URL to redirect to after OAuth flow completes (only used for OAuth flows)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.create(
                short_name="short_name",
                readable_collection_id="readable_collection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            short_name=short_name,
            readable_collection_id=readable_collection_id,
            name=name,
            description=description,
            config=config,
            schedule=schedule,
            sync_immediately=sync_immediately,
            authentication=authentication,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, source_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnection:
        """
        Get a source connection with optional depth expansion.

        Parameters
        ----------
        source_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.get(
                source_connection_id="source_connection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(source_connection_id, request_options=request_options)
        return _response.data

    async def delete(
        self, source_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnection:
        """
        Delete a source connection and all related data.

        Parameters
        ----------
        source_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.delete(
                source_connection_id="source_connection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(source_connection_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        source_connection_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        schedule: typing.Optional[ScheduleConfig] = OMIT,
        authentication: typing.Optional[Authentication] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Update a source connection.

        Updateable fields:
        - name, description
        - config_fields
        - cron_schedule
        - auth_fields (direct auth only)

        Parameters
        ----------
        source_connection_id : str

        name : typing.Optional[str]

        description : typing.Optional[str]

        config : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Source-specific configuration

        schedule : typing.Optional[ScheduleConfig]

        authentication : typing.Optional[Authentication]
            Authentication config (defaults to OAuth browser flow for OAuth sources)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.update(
                source_connection_id="source_connection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            source_connection_id,
            name=name,
            description=description,
            config=config,
            schedule=schedule,
            authentication=authentication,
            request_options=request_options,
        )
        return _response.data

    async def run(
        self,
        source_connection_id: str,
        *,
        force_full_sync: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnectionJob:
        """
        Trigger a sync run for a source connection.

        Runs are always executed through Temporal workflow engine.

        Args:
            db: Database session
            source_connection_id: ID of the source connection to run
            ctx: API context with organization and user information
            guard_rail: Guard rail service for usage limits
            force_full_sync: If True, forces a full sync with orphaned entity cleanup
                            for continuous syncs. Raises 400 error if used on
                            non-continuous syncs (which are always full syncs).

        Parameters
        ----------
        source_connection_id : str

        force_full_sync : typing.Optional[bool]
            Force a full sync ignoring cursor data instead of waiting for the daily cleanup schedule. Only allowed for continuous syncs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnectionJob
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.run(
                source_connection_id="source_connection_id",
                force_full_sync=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.run(
            source_connection_id, force_full_sync=force_full_sync, request_options=request_options
        )
        return _response.data

    async def get_source_connection_jobs(
        self,
        source_connection_id: str,
        *,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SourceConnectionJob]:
        """
        Get sync jobs for a source connection.

        Parameters
        ----------
        source_connection_id : str

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionJob]
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.get_source_connection_jobs(
                source_connection_id="source_connection_id",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_source_connection_jobs(
            source_connection_id, limit=limit, request_options=request_options
        )
        return _response.data

    async def cancel_job(
        self, source_connection_id: str, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SourceConnectionJob:
        """
        Cancel a running sync job for a source connection.

        This endpoint requests cancellation and marks the job as CANCELLING.
        The workflow updates the final status to CANCELLED when it processes
        the cancellation request.

        Parameters
        ----------
        source_connection_id : str

        job_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnectionJob
            Successful Response

        Examples
        --------
        import asyncio

        from graffo import AsyncGraffoSDK

        client = AsyncGraffoSDK(
            framework_name="YOUR_FRAMEWORK_NAME",
            framework_version="YOUR_FRAMEWORK_VERSION",
            agent_key="YOUR_AGENT_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
            agent_key="YOUR_AGENT_KEY",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.source_connections.cancel_job(
                source_connection_id="source_connection_id",
                job_id="job_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel_job(source_connection_id, job_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .config_values import ConfigValues


class AuthProviderConnectionCreate(UniversalBaseModel):
    """
    Schema for creating an auth provider connection with credentials.
    """

    name: str = pydantic.Field()
    """
    Human-readable name for this auth provider connection
    """

    readable_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL-safe unique identifier for the connection. Must contain only lowercase letters, numbers, and hyphens. If not provided, it will be automatically generated from the connection name with a random suffix for uniqueness (e.g., 'composio-connection-ab123').
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional detailed description of what this auth provider connection provides.
    """

    short_name: str = pydantic.Field()
    """
    Technical identifier of the auth provider
    """

    auth_fields: typing.Optional[ConfigValues] = pydantic.Field(default=None)
    """
    Authentication credentials required to access the auth provider. The required fields vary by auth provider type.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .authentication_method import AuthenticationMethod
from .source_connection_status import SourceConnectionStatus


class SourceConnectionListItem(UniversalBaseModel):
    """
    Clean source connection for list views.
    """

    id: str
    name: str
    short_name: str
    readable_collection_id: str
    created_at: dt.datetime
    modified_at: dt.datetime
    is_authenticated: bool
    entity_count: typing.Optional[int] = None
    federated_search: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this source uses federated search
    """

    auth_method: AuthenticationMethod = pydantic.Field()
    """
    Get authentication method from database value.
    """

    status: SourceConnectionStatus = pydantic.Field()
    """
    Compute connection status from current state.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audit_log_category import AuditLogCategory
from .audit_log_level import AuditLogLevel


class AuditLog(UniversalBaseModel):
    """
    Audit log response schema.
    """

    category: AuditLogCategory
    level: AuditLogLevel
    action: str
    details: str
    actor: str
    actor_email: typing.Optional[str] = None
    actor_type: typing.Optional[str] = None
    user_id: typing.Optional[str] = None
    api_key_id: typing.Optional[str] = None
    agent_api_key_id: typing.Optional[str] = None
    search_query_id: typing.Optional[str] = None
    sync_job_id: typing.Optional[str] = None
    status_code: typing.Optional[int] = None
    ip_address: typing.Optional[str] = None
    event_metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    trace: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    id: str
    organization_id: str
    collection_id: str
    created_at: typing.Optional[dt.datetime] = None
    modified_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AuditLogCategory = typing.Union[
    typing.Literal["Auth & Security", "Data Sync", "RAG Trace", "API/MCP Usage", "System"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SyncJobAuditSummary(UniversalBaseModel):
    """
    Summary of audit entries for a sync job.
    """

    sync_job_id: str
    total_changes: int
    action_counts: typing.Dict[str, int]
    items_created: int
    items_updated: int
    items_deleted: int
    errors_encountered: int
    processing_time_stats: typing.Optional[typing.Dict[str, typing.Optional[float]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .agent_api_key_status import AgentApiKeyStatus


class AgentApiKeySummary(UniversalBaseModel):
    """
    Lightweight agent API key summary for lists and selectors.
    """

    id: str
    name: str
    key_prefix: str
    key_hint: str
    status: AgentApiKeyStatus
    cluster_count: typing.Optional[int] = None
    is_read_only: typing.Optional[bool] = None
    last_used_at: typing.Optional[str] = None
    expires_at: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs


class MinShould(UniversalBaseModel):
    conditions: typing.List["MinShouldConditionsItem"] = pydantic.Field()
    """
    
    """

    min_count: int = pydantic.Field()
    """
    
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .filter import Filter  # noqa: E402, F401, I001
from .nested import Nested  # noqa: E402, F401, I001
from .nested_condition import NestedCondition  # noqa: E402, F401, I001
from .min_should_conditions_item import MinShouldConditionsItem  # noqa: E402, F401, I001

update_forward_refs(MinShould)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audit_log import AuditLog
from .search_query_response import SearchQueryResponse
from .source_distribution import SourceDistribution


class DashboardStats(UniversalBaseModel):
    total_assets_indexed: int
    active_agents: int
    active_contexts: int
    total_queries: int
    sync_success_rate: float
    stale_item_count: int
    knowledge_gaps: typing.List[SearchQueryResponse]
    recent_uploads: typing.List[AuditLog]
    source_distribution: typing.List[SourceDistribution]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SearchQueryResponse(UniversalBaseModel):
    """
    Schema for search query responses.
    """

    query_text: str = pydantic.Field()
    """
    The search query text
    """

    query_length: int = pydantic.Field()
    """
    Length of the search query in characters
    """

    is_streaming: bool = pydantic.Field()
    """
    Whether this was a streaming search
    """

    retrieval_strategy: str = pydantic.Field()
    """
    Retrieval strategy: 'hybrid', 'neural', 'keyword'
    """

    limit: int = pydantic.Field()
    """
    Maximum number of results requested
    """

    offset: int = pydantic.Field()
    """
    Number of results to skip for pagination
    """

    temporal_relevance: float = pydantic.Field()
    """
    Temporal relevance weight (0.0 to 1.0)
    """

    filter: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Qdrant filter applied (if any)
    """

    duration_ms: int = pydantic.Field()
    """
    Search execution time in milliseconds
    """

    results_count: int = pydantic.Field()
    """
    Number of results returned
    """

    max_score: typing.Optional[float] = pydantic.Field(default=None)
    """
    Highest relevance score from search results (0.0 to 1.0)
    """

    expand_query: bool = pydantic.Field()
    """
    Whether query expansion was enabled
    """

    interpret_filters: bool = pydantic.Field()
    """
    Whether query interpretation was enabled
    """

    rerank: bool = pydantic.Field()
    """
    Whether LLM reranking was enabled
    """

    generate_answer: bool = pydantic.Field()
    """
    Whether answer generation was enabled
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the search query
    """

    organization_id: str = pydantic.Field()
    """
    ID of the organization
    """

    collection_id: str = pydantic.Field()
    """
    ID of the collection that was searched
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the user who performed the search
    """

    api_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the API key used for the search
    """

    created_at: str = pydantic.Field()
    """
    When the search query was created
    """

    modified_at: str = pydantic.Field()
    """
    When the search query was last modified
    """

    created_by_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email of the user who created the record
    """

    modified_by_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email of the user who last modified the record
    """

    confidence: typing.Optional[float] = pydantic.Field(default=None)
    """
    Confidence score of the search results
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AgentApiKeyCreate(UniversalBaseModel):
    """
    Schema for creating a new agent API key.
    """

    name: str
    description: typing.Optional[str] = None
    is_read_only: typing.Optional[bool] = None
    rate_limit_per_minute: typing.Optional[int] = None
    rate_limit_per_hour: typing.Optional[int] = None
    rate_limit_per_day: typing.Optional[int] = None
    auto_renew: typing.Optional[bool] = None
    collection_id: str
    context_cluster_ids: typing.Optional[typing.List[str]] = None
    expires_at: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AuthenticationMethod = typing.Union[
    typing.Literal["direct", "oauth_browser", "oauth_token", "oauth_byoc", "auth_provider"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class OrganizationCreationValidation(UniversalBaseModel):
    """
    Response with validation results for organization creation.
    """

    can_create: bool = pydantic.Field()
    """
    Whether user can create organization with this plan
    """

    reason: typing.Optional[str] = pydantic.Field(default=None)
    """
    Reason if creation is not allowed
    """

    available_plans: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of available billing plans
    """

    has_free_organization: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether user already has a free/developer organization
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BillingPeriodStatus = typing.Union[typing.Literal["active", "completed", "ended_unpaid", "trial", "grace"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class BulkOperationResult(UniversalBaseModel):
    """
    Result of bulk operations.
    """

    updated_count: int
    failed_count: int
    warnings: typing.Optional[typing.List[str]] = None
    audit_entries_created: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .feature_flag import FeatureFlag


class OrganizationMetrics(UniversalBaseModel):
    """
    Comprehensive organization metrics for admin dashboard.

    Combines organization info with billing and usage metrics from the Usage model.
    """

    id: str = pydantic.Field()
    """
    Organization ID
    """

    name: str = pydantic.Field()
    """
    Organization name
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization description
    """

    created_at: dt.datetime = pydantic.Field()
    """
    When organization was created
    """

    modified_at: dt.datetime = pydantic.Field()
    """
    Last modification time
    """

    auth_0_org_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="auth0_org_id")] = (
        pydantic.Field(default=None)
    )
    """
    Auth0 organization ID
    """

    billing_plan: typing.Optional[str] = pydantic.Field(default=None)
    """
    Current billing plan (trial, starter, pro, enterprise)
    """

    billing_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Billing status (active, cancelled, past_due, etc.)
    """

    stripe_customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Stripe customer ID
    """

    trial_ends_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When trial ends
    """

    user_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of users in organization
    """

    source_connection_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of source connections (computed from source_connection table)
    """

    entity_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of entities (from Usage.entities)
    """

    query_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of queries (from Usage.queries)
    """

    last_active_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last active timestamp of any user in this organization
    """

    is_member: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the current admin user is already a member
    """

    member_role: typing.Optional[str] = pydantic.Field(default=None)
    """
    Admin's role in this organization (if member)
    """

    enabled_features: typing.Optional[typing.List[FeatureFlag]] = pydantic.Field(default=None)
    """
    List of enabled feature flags for this organization
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs


class CatalogTreeView(UniversalBaseModel):
    """
    Complete tree view for a collection's catalog.
    """

    collection_id: str
    root_nodes: typing.List["CatalogTreeNode"]
    total_items: int
    selected_items: int
    total_chunks: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .catalog_tree_node import CatalogTreeNode  # noqa: E402, F401, I001

update_forward_refs(CatalogTreeView)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UsageTrend(UniversalBaseModel):
    """
    Usage trend information.
    """

    metric: str = pydantic.Field()
    """
    Metric name (syncs, entities, etc)
    """

    direction: str = pydantic.Field()
    """
    Trend direction: up, down, or stable
    """

    percentage_change: float = pydantic.Field()
    """
    Percentage change from previous period
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .action_check_request import ActionCheckRequest
    from .action_check_response import ActionCheckResponse
    from .agent_api_key import AgentApiKey
    from .agent_api_key_cluster_management import AgentApiKeyClusterManagement
    from .agent_api_key_collection_stats import AgentApiKeyCollectionStats
    from .agent_api_key_create import AgentApiKeyCreate
    from .agent_api_key_revoke import AgentApiKeyRevoke
    from .agent_api_key_status import AgentApiKeyStatus
    from .agent_api_key_summary import AgentApiKeySummary
    from .agent_api_key_update import AgentApiKeyUpdate
    from .agent_api_key_with_clusters import AgentApiKeyWithClusters
    from .agent_api_key_with_full_key import AgentApiKeyWithFullKey
    from .any import Any
    from .api_key import ApiKey
    from .api_key_create import ApiKeyCreate
    from .audit_action_type import AuditActionType
    from .audit_log import AuditLog
    from .audit_log_category import AuditLogCategory
    from .audit_log_level import AuditLogLevel
    from .audit_log_search_result import AuditLogSearchResult
    from .auth_provider import AuthProvider
    from .auth_provider_authentication import AuthProviderAuthentication
    from .auth_provider_connection import AuthProviderConnection
    from .auth_provider_connection_create import AuthProviderConnectionCreate
    from .auth_provider_connection_update import AuthProviderConnectionUpdate
    from .authentication_details import AuthenticationDetails
    from .authentication_method import AuthenticationMethod
    from .billing_period import BillingPeriod
    from .billing_period_status import BillingPeriodStatus
    from .billing_period_usage import BillingPeriodUsage
    from .billing_plan import BillingPlan
    from .billing_status import BillingStatus
    from .billing_transition import BillingTransition
    from .body_connect_slack_with_token_connections_direct_token_slack_post import (
        BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost,
    )
    from .bulk_operation_result import BulkOperationResult
    from .bulk_rag_update import BulkRagUpdate
    from .bulk_sensitivity_update import BulkSensitivityUpdate
    from .bulk_status_update import BulkStatusUpdate
    from .catalog_audit_log import CatalogAuditLog
    from .catalog_audit_log_summary import CatalogAuditLogSummary
    from .catalog_collection_stats import CatalogCollectionStats
    from .catalog_item import CatalogItem
    from .catalog_item_governance_update import CatalogItemGovernanceUpdate
    from .catalog_item_summary import CatalogItemSummary
    from .catalog_item_with_governance import CatalogItemWithGovernance
    from .catalog_status import CatalogStatus
    from .catalog_tree_node import CatalogTreeNode
    from .catalog_tree_view import CatalogTreeView
    from .checkout_session_request import CheckoutSessionRequest
    from .checkout_session_response import CheckoutSessionResponse
    from .collection import Collection
    from .collection_member import CollectionMember
    from .collection_member_add import CollectionMemberAdd
    from .collection_role import CollectionRole
    from .collection_status import CollectionStatus
    from .collection_update import CollectionUpdate
    from .compliance_report import ComplianceReport
    from .config_field import ConfigField
    from .config_values import ConfigValues
    from .connection import Connection
    from .connection_status import ConnectionStatus
    from .content_update_stats import ContentUpdateStats
    from .context_cluster import ContextCluster
    from .context_cluster_collection_stats import ContextClusterCollectionStats
    from .context_cluster_create import ContextClusterCreate
    from .context_cluster_item_management import ContextClusterItemManagement
    from .context_cluster_status import ContextClusterStatus
    from .context_cluster_summary import ContextClusterSummary
    from .context_cluster_update import ContextClusterUpdate
    from .context_cluster_with_items import ContextClusterWithItems
    from .customer_portal_request import CustomerPortalRequest
    from .customer_portal_response import CustomerPortalResponse
    from .dashboard_stats import DashboardStats
    from .datetime_range import DatetimeRange
    from .destination import Destination
    from .destination_with_authentication_fields import DestinationWithAuthenticationFields
    from .direct_authentication import DirectAuthentication
    from .embedding_model import EmbeddingModel
    from .embedding_model_with_authentication_fields import EmbeddingModelWithAuthenticationFields
    from .entity_count import EntityCount
    from .entity_count_with_definition import EntityCountWithDefinition
    from .entity_definition import EntityDefinition
    from .entity_definition_create import EntityDefinitionCreate
    from .entity_definition_create_entity_schema import EntityDefinitionCreateEntitySchema
    from .entity_definition_entity_schema import EntityDefinitionEntitySchema
    from .entity_summary import EntitySummary
    from .entity_type import EntityType
    from .entity_type_stats import EntityTypeStats
    from .except_ import Except
    from .feature_flag import FeatureFlag
    from .field_condition import FieldCondition
    from .fields import Fields
    from .filter import Filter
    from .filter_must_item import FilterMustItem
    from .filter_must_not import FilterMustNot
    from .filter_must_not_item import FilterMustNotItem
    from .filter_should_item import FilterShouldItem
    from .geo_bounding_box import GeoBoundingBox
    from .geo_line_string import GeoLineString
    from .geo_point import GeoPoint
    from .geo_polygon import GeoPolygon
    from .geo_radius import GeoRadius
    from .governance_activity_feed import GovernanceActivityFeed
    from .graph_link import GraphLink
    from .graph_node import GraphNode
    from .graph_response import GraphResponse
    from .gt import Gt
    from .gte import Gte
    from .has_id_condition import HasIdCondition
    from .has_id_condition_has_id_item import HasIdConditionHasIdItem
    from .has_vector_condition import HasVectorCondition
    from .http_validation_error import HttpValidationError
    from .integration_credential_in_db import IntegrationCredentialInDb
    from .integration_credential_raw_create import IntegrationCredentialRawCreate
    from .integration_type import IntegrationType
    from .invitation_create import InvitationCreate
    from .invitation_response import InvitationResponse
    from .is_empty_condition import IsEmptyCondition
    from .is_null_condition import IsNullCondition
    from .legacy_search_request import LegacySearchRequest
    from .legacy_search_request_search_method import LegacySearchRequestSearchMethod
    from .legacy_search_response import LegacySearchResponse
    from .lt import Lt
    from .lte import Lte
    from .match import Match
    from .match_any import MatchAny
    from .match_except import MatchExcept
    from .match_phrase import MatchPhrase
    from .match_text import MatchText
    from .match_text_any import MatchTextAny
    from .match_value import MatchValue
    from .member_response import MemberResponse
    from .message_response import MessageResponse
    from .min_should import MinShould
    from .min_should_conditions_item import MinShouldConditionsItem
    from .minute_level_schedule_config import MinuteLevelScheduleConfig
    from .must import Must
    from .nested import Nested
    from .nested_condition import NestedCondition
    from .o_auth_browser_authentication import OAuthBrowserAuthentication
    from .o_auth_token_authentication import OAuthTokenAuthentication
    from .o_auth_type import OAuthType
    from .organization import Organization
    from .organization_billing import OrganizationBilling
    from .organization_create import OrganizationCreate
    from .organization_creation_validation import OrganizationCreationValidation
    from .organization_creation_validation_request import OrganizationCreationValidationRequest
    from .organization_metrics import OrganizationMetrics
    from .organization_readiness_response import OrganizationReadinessResponse
    from .organization_with_role import OrganizationWithRole
    from .payload_field import PayloadField
    from .query_expansion_strategy import QueryExpansionStrategy
    from .rag_entity_list import RagEntityList
    from .rag_governance_report import RagGovernanceReport
    from .range import Range
    from .response_type import ResponseType
    from .retrieval_strategy import RetrievalStrategy
    from .s_3_config_request import S3ConfigRequest
    from .s_3_config_response import S3ConfigResponse
    from .schedule_config import ScheduleConfig
    from .schedule_details import ScheduleDetails
    from .schedule_response import ScheduleResponse
    from .search_query_response import SearchQueryResponse
    from .search_request import SearchRequest
    from .search_response import SearchResponse
    from .search_status import SearchStatus
    from .sensitivity_level import SensitivityLevel
    from .should import Should
    from .single_action_check_response import SingleActionCheckResponse
    from .single_action_check_response_reason import SingleActionCheckResponseReason
    from .source import Source
    from .source_connection import SourceConnection
    from .source_connection_job import SourceConnectionJob
    from .source_connection_list_item import SourceConnectionListItem
    from .source_connection_status import SourceConnectionStatus
    from .source_distribution import SourceDistribution
    from .source_rate_limit import SourceRateLimit
    from .source_rate_limit_response import SourceRateLimitResponse
    from .source_rate_limit_update_request import SourceRateLimitUpdateRequest
    from .subscription_info import SubscriptionInfo
    from .sync import Sync
    from .sync_create import SyncCreate
    from .sync_details import SyncDetails
    from .sync_job import SyncJob
    from .sync_job_audit_summary import SyncJobAuditSummary
    from .sync_job_details import SyncJobDetails
    from .sync_job_status import SyncJobStatus
    from .sync_status import SyncStatus
    from .sync_update import SyncUpdate
    from .sync_with_source_connection import SyncWithSourceConnection
    from .transformer import Transformer
    from .transformer_create import TransformerCreate
    from .transformer_update import TransformerUpdate
    from .update_plan_request import UpdatePlanRequest
    from .usage_dashboard import UsageDashboard
    from .usage_snapshot import UsageSnapshot
    from .usage_trend import UsageTrend
    from .user import User
    from .user_activity_summary import UserActivitySummary
    from .user_create import UserCreate
    from .user_organization import UserOrganization
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
    from .value import Value
    from .values_count import ValuesCount
_dynamic_imports: typing.Dict[str, str] = {
    "ActionCheckRequest": ".action_check_request",
    "ActionCheckResponse": ".action_check_response",
    "AgentApiKey": ".agent_api_key",
    "AgentApiKeyClusterManagement": ".agent_api_key_cluster_management",
    "AgentApiKeyCollectionStats": ".agent_api_key_collection_stats",
    "AgentApiKeyCreate": ".agent_api_key_create",
    "AgentApiKeyRevoke": ".agent_api_key_revoke",
    "AgentApiKeyStatus": ".agent_api_key_status",
    "AgentApiKeySummary": ".agent_api_key_summary",
    "AgentApiKeyUpdate": ".agent_api_key_update",
    "AgentApiKeyWithClusters": ".agent_api_key_with_clusters",
    "AgentApiKeyWithFullKey": ".agent_api_key_with_full_key",
    "Any": ".any",
    "ApiKey": ".api_key",
    "ApiKeyCreate": ".api_key_create",
    "AuditActionType": ".audit_action_type",
    "AuditLog": ".audit_log",
    "AuditLogCategory": ".audit_log_category",
    "AuditLogLevel": ".audit_log_level",
    "AuditLogSearchResult": ".audit_log_search_result",
    "AuthProvider": ".auth_provider",
    "AuthProviderAuthentication": ".auth_provider_authentication",
    "AuthProviderConnection": ".auth_provider_connection",
    "AuthProviderConnectionCreate": ".auth_provider_connection_create",
    "AuthProviderConnectionUpdate": ".auth_provider_connection_update",
    "AuthenticationDetails": ".authentication_details",
    "AuthenticationMethod": ".authentication_method",
    "BillingPeriod": ".billing_period",
    "BillingPeriodStatus": ".billing_period_status",
    "BillingPeriodUsage": ".billing_period_usage",
    "BillingPlan": ".billing_plan",
    "BillingStatus": ".billing_status",
    "BillingTransition": ".billing_transition",
    "BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost": ".body_connect_slack_with_token_connections_direct_token_slack_post",
    "BulkOperationResult": ".bulk_operation_result",
    "BulkRagUpdate": ".bulk_rag_update",
    "BulkSensitivityUpdate": ".bulk_sensitivity_update",
    "BulkStatusUpdate": ".bulk_status_update",
    "CatalogAuditLog": ".catalog_audit_log",
    "CatalogAuditLogSummary": ".catalog_audit_log_summary",
    "CatalogCollectionStats": ".catalog_collection_stats",
    "CatalogItem": ".catalog_item",
    "CatalogItemGovernanceUpdate": ".catalog_item_governance_update",
    "CatalogItemSummary": ".catalog_item_summary",
    "CatalogItemWithGovernance": ".catalog_item_with_governance",
    "CatalogStatus": ".catalog_status",
    "CatalogTreeNode": ".catalog_tree_node",
    "CatalogTreeView": ".catalog_tree_view",
    "CheckoutSessionRequest": ".checkout_session_request",
    "CheckoutSessionResponse": ".checkout_session_response",
    "Collection": ".collection",
    "CollectionMember": ".collection_member",
    "CollectionMemberAdd": ".collection_member_add",
    "CollectionRole": ".collection_role",
    "CollectionStatus": ".collection_status",
    "CollectionUpdate": ".collection_update",
    "ComplianceReport": ".compliance_report",
    "ConfigField": ".config_field",
    "ConfigValues": ".config_values",
    "Connection": ".connection",
    "ConnectionStatus": ".connection_status",
    "ContentUpdateStats": ".content_update_stats",
    "ContextCluster": ".context_cluster",
    "ContextClusterCollectionStats": ".context_cluster_collection_stats",
    "ContextClusterCreate": ".context_cluster_create",
    "ContextClusterItemManagement": ".context_cluster_item_management",
    "ContextClusterStatus": ".context_cluster_status",
    "ContextClusterSummary": ".context_cluster_summary",
    "ContextClusterUpdate": ".context_cluster_update",
    "ContextClusterWithItems": ".context_cluster_with_items",
    "CustomerPortalRequest": ".customer_portal_request",
    "CustomerPortalResponse": ".customer_portal_response",
    "DashboardStats": ".dashboard_stats",
    "DatetimeRange": ".datetime_range",
    "Destination": ".destination",
    "DestinationWithAuthenticationFields": ".destination_with_authentication_fields",
    "DirectAuthentication": ".direct_authentication",
    "EmbeddingModel": ".embedding_model",
    "EmbeddingModelWithAuthenticationFields": ".embedding_model_with_authentication_fields",
    "EntityCount": ".entity_count",
    "EntityCountWithDefinition": ".entity_count_with_definition",
    "EntityDefinition": ".entity_definition",
    "EntityDefinitionCreate": ".entity_definition_create",
    "EntityDefinitionCreateEntitySchema": ".entity_definition_create_entity_schema",
    "EntityDefinitionEntitySchema": ".entity_definition_entity_schema",
    "EntitySummary": ".entity_summary",
    "EntityType": ".entity_type",
    "EntityTypeStats": ".entity_type_stats",
    "Except": ".except_",
    "FeatureFlag": ".feature_flag",
    "FieldCondition": ".field_condition",
    "Fields": ".fields",
    "Filter": ".filter",
    "FilterMustItem": ".filter_must_item",
    "FilterMustNot": ".filter_must_not",
    "FilterMustNotItem": ".filter_must_not_item",
    "FilterShouldItem": ".filter_should_item",
    "GeoBoundingBox": ".geo_bounding_box",
    "GeoLineString": ".geo_line_string",
    "GeoPoint": ".geo_point",
    "GeoPolygon": ".geo_polygon",
    "GeoRadius": ".geo_radius",
    "GovernanceActivityFeed": ".governance_activity_feed",
    "GraphLink": ".graph_link",
    "GraphNode": ".graph_node",
    "GraphResponse": ".graph_response",
    "Gt": ".gt",
    "Gte": ".gte",
    "HasIdCondition": ".has_id_condition",
    "HasIdConditionHasIdItem": ".has_id_condition_has_id_item",
    "HasVectorCondition": ".has_vector_condition",
    "HttpValidationError": ".http_validation_error",
    "IntegrationCredentialInDb": ".integration_credential_in_db",
    "IntegrationCredentialRawCreate": ".integration_credential_raw_create",
    "IntegrationType": ".integration_type",
    "InvitationCreate": ".invitation_create",
    "InvitationResponse": ".invitation_response",
    "IsEmptyCondition": ".is_empty_condition",
    "IsNullCondition": ".is_null_condition",
    "LegacySearchRequest": ".legacy_search_request",
    "LegacySearchRequestSearchMethod": ".legacy_search_request_search_method",
    "LegacySearchResponse": ".legacy_search_response",
    "Lt": ".lt",
    "Lte": ".lte",
    "Match": ".match",
    "MatchAny": ".match_any",
    "MatchExcept": ".match_except",
    "MatchPhrase": ".match_phrase",
    "MatchText": ".match_text",
    "MatchTextAny": ".match_text_any",
    "MatchValue": ".match_value",
    "MemberResponse": ".member_response",
    "MessageResponse": ".message_response",
    "MinShould": ".min_should",
    "MinShouldConditionsItem": ".min_should_conditions_item",
    "MinuteLevelScheduleConfig": ".minute_level_schedule_config",
    "Must": ".must",
    "Nested": ".nested",
    "NestedCondition": ".nested_condition",
    "OAuthBrowserAuthentication": ".o_auth_browser_authentication",
    "OAuthTokenAuthentication": ".o_auth_token_authentication",
    "OAuthType": ".o_auth_type",
    "Organization": ".organization",
    "OrganizationBilling": ".organization_billing",
    "OrganizationCreate": ".organization_create",
    "OrganizationCreationValidation": ".organization_creation_validation",
    "OrganizationCreationValidationRequest": ".organization_creation_validation_request",
    "OrganizationMetrics": ".organization_metrics",
    "OrganizationReadinessResponse": ".organization_readiness_response",
    "OrganizationWithRole": ".organization_with_role",
    "PayloadField": ".payload_field",
    "QueryExpansionStrategy": ".query_expansion_strategy",
    "RagEntityList": ".rag_entity_list",
    "RagGovernanceReport": ".rag_governance_report",
    "Range": ".range",
    "ResponseType": ".response_type",
    "RetrievalStrategy": ".retrieval_strategy",
    "S3ConfigRequest": ".s_3_config_request",
    "S3ConfigResponse": ".s_3_config_response",
    "ScheduleConfig": ".schedule_config",
    "ScheduleDetails": ".schedule_details",
    "ScheduleResponse": ".schedule_response",
    "SearchQueryResponse": ".search_query_response",
    "SearchRequest": ".search_request",
    "SearchResponse": ".search_response",
    "SearchStatus": ".search_status",
    "SensitivityLevel": ".sensitivity_level",
    "Should": ".should",
    "SingleActionCheckResponse": ".single_action_check_response",
    "SingleActionCheckResponseReason": ".single_action_check_response_reason",
    "Source": ".source",
    "SourceConnection": ".source_connection",
    "SourceConnectionJob": ".source_connection_job",
    "SourceConnectionListItem": ".source_connection_list_item",
    "SourceConnectionStatus": ".source_connection_status",
    "SourceDistribution": ".source_distribution",
    "SourceRateLimit": ".source_rate_limit",
    "SourceRateLimitResponse": ".source_rate_limit_response",
    "SourceRateLimitUpdateRequest": ".source_rate_limit_update_request",
    "SubscriptionInfo": ".subscription_info",
    "Sync": ".sync",
    "SyncCreate": ".sync_create",
    "SyncDetails": ".sync_details",
    "SyncJob": ".sync_job",
    "SyncJobAuditSummary": ".sync_job_audit_summary",
    "SyncJobDetails": ".sync_job_details",
    "SyncJobStatus": ".sync_job_status",
    "SyncStatus": ".sync_status",
    "SyncUpdate": ".sync_update",
    "SyncWithSourceConnection": ".sync_with_source_connection",
    "Transformer": ".transformer",
    "TransformerCreate": ".transformer_create",
    "TransformerUpdate": ".transformer_update",
    "UpdatePlanRequest": ".update_plan_request",
    "UsageDashboard": ".usage_dashboard",
    "UsageSnapshot": ".usage_snapshot",
    "UsageTrend": ".usage_trend",
    "User": ".user",
    "UserActivitySummary": ".user_activity_summary",
    "UserCreate": ".user_create",
    "UserOrganization": ".user_organization",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
    "Value": ".value",
    "ValuesCount": ".values_count",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ActionCheckRequest",
    "ActionCheckResponse",
    "AgentApiKey",
    "AgentApiKeyClusterManagement",
    "AgentApiKeyCollectionStats",
    "AgentApiKeyCreate",
    "AgentApiKeyRevoke",
    "AgentApiKeyStatus",
    "AgentApiKeySummary",
    "AgentApiKeyUpdate",
    "AgentApiKeyWithClusters",
    "AgentApiKeyWithFullKey",
    "Any",
    "ApiKey",
    "ApiKeyCreate",
    "AuditActionType",
    "AuditLog",
    "AuditLogCategory",
    "AuditLogLevel",
    "AuditLogSearchResult",
    "AuthProvider",
    "AuthProviderAuthentication",
    "AuthProviderConnection",
    "AuthProviderConnectionCreate",
    "AuthProviderConnectionUpdate",
    "AuthenticationDetails",
    "AuthenticationMethod",
    "BillingPeriod",
    "BillingPeriodStatus",
    "BillingPeriodUsage",
    "BillingPlan",
    "BillingStatus",
    "BillingTransition",
    "BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost",
    "BulkOperationResult",
    "BulkRagUpdate",
    "BulkSensitivityUpdate",
    "BulkStatusUpdate",
    "CatalogAuditLog",
    "CatalogAuditLogSummary",
    "CatalogCollectionStats",
    "CatalogItem",
    "CatalogItemGovernanceUpdate",
    "CatalogItemSummary",
    "CatalogItemWithGovernance",
    "CatalogStatus",
    "CatalogTreeNode",
    "CatalogTreeView",
    "CheckoutSessionRequest",
    "CheckoutSessionResponse",
    "Collection",
    "CollectionMember",
    "CollectionMemberAdd",
    "CollectionRole",
    "CollectionStatus",
    "CollectionUpdate",
    "ComplianceReport",
    "ConfigField",
    "ConfigValues",
    "Connection",
    "ConnectionStatus",
    "ContentUpdateStats",
    "ContextCluster",
    "ContextClusterCollectionStats",
    "ContextClusterCreate",
    "ContextClusterItemManagement",
    "ContextClusterStatus",
    "ContextClusterSummary",
    "ContextClusterUpdate",
    "ContextClusterWithItems",
    "CustomerPortalRequest",
    "CustomerPortalResponse",
    "DashboardStats",
    "DatetimeRange",
    "Destination",
    "DestinationWithAuthenticationFields",
    "DirectAuthentication",
    "EmbeddingModel",
    "EmbeddingModelWithAuthenticationFields",
    "EntityCount",
    "EntityCountWithDefinition",
    "EntityDefinition",
    "EntityDefinitionCreate",
    "EntityDefinitionCreateEntitySchema",
    "EntityDefinitionEntitySchema",
    "EntitySummary",
    "EntityType",
    "EntityTypeStats",
    "Except",
    "FeatureFlag",
    "FieldCondition",
    "Fields",
    "Filter",
    "FilterMustItem",
    "FilterMustNot",
    "FilterMustNotItem",
    "FilterShouldItem",
    "GeoBoundingBox",
    "GeoLineString",
    "GeoPoint",
    "GeoPolygon",
    "GeoRadius",
    "GovernanceActivityFeed",
    "GraphLink",
    "GraphNode",
    "GraphResponse",
    "Gt",
    "Gte",
    "HasIdCondition",
    "HasIdConditionHasIdItem",
    "HasVectorCondition",
    "HttpValidationError",
    "IntegrationCredentialInDb",
    "IntegrationCredentialRawCreate",
    "IntegrationType",
    "InvitationCreate",
    "InvitationResponse",
    "IsEmptyCondition",
    "IsNullCondition",
    "LegacySearchRequest",
    "LegacySearchRequestSearchMethod",
    "LegacySearchResponse",
    "Lt",
    "Lte",
    "Match",
    "MatchAny",
    "MatchExcept",
    "MatchPhrase",
    "MatchText",
    "MatchTextAny",
    "MatchValue",
    "MemberResponse",
    "MessageResponse",
    "MinShould",
    "MinShouldConditionsItem",
    "MinuteLevelScheduleConfig",
    "Must",
    "Nested",
    "NestedCondition",
    "OAuthBrowserAuthentication",
    "OAuthTokenAuthentication",
    "OAuthType",
    "Organization",
    "OrganizationBilling",
    "OrganizationCreate",
    "OrganizationCreationValidation",
    "OrganizationCreationValidationRequest",
    "OrganizationMetrics",
    "OrganizationReadinessResponse",
    "OrganizationWithRole",
    "PayloadField",
    "QueryExpansionStrategy",
    "RagEntityList",
    "RagGovernanceReport",
    "Range",
    "ResponseType",
    "RetrievalStrategy",
    "S3ConfigRequest",
    "S3ConfigResponse",
    "ScheduleConfig",
    "ScheduleDetails",
    "ScheduleResponse",
    "SearchQueryResponse",
    "SearchRequest",
    "SearchResponse",
    "SearchStatus",
    "SensitivityLevel",
    "Should",
    "SingleActionCheckResponse",
    "SingleActionCheckResponseReason",
    "Source",
    "SourceConnection",
    "SourceConnectionJob",
    "SourceConnectionListItem",
    "SourceConnectionStatus",
    "SourceDistribution",
    "SourceRateLimit",
    "SourceRateLimitResponse",
    "SourceRateLimitUpdateRequest",
    "SubscriptionInfo",
    "Sync",
    "SyncCreate",
    "SyncDetails",
    "SyncJob",
    "SyncJobAuditSummary",
    "SyncJobDetails",
    "SyncJobStatus",
    "SyncStatus",
    "SyncUpdate",
    "SyncWithSourceConnection",
    "Transformer",
    "TransformerCreate",
    "TransformerUpdate",
    "UpdatePlanRequest",
    "UsageDashboard",
    "UsageSnapshot",
    "UsageTrend",
    "User",
    "UserActivitySummary",
    "UserCreate",
    "UserOrganization",
    "ValidationError",
    "ValidationErrorLocItem",
    "Value",
    "ValuesCount",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .billing_period import BillingPeriod
from .billing_plan import BillingPlan
from .billing_status import BillingStatus


class OrganizationBilling(UniversalBaseModel):
    """
    Organization billing schema with current period information.

    This schema is enriched with the current active billing period for
    efficient access to rate limits and billing status without additional queries.
    """

    billing_plan: typing.Optional[BillingPlan] = pydantic.Field(default=None)
    """
    Current billing plan
    """

    billing_status: typing.Optional[BillingStatus] = pydantic.Field(default=None)
    """
    Current billing status
    """

    billing_email: str = pydantic.Field()
    """
    Billing contact email
    """

    id: str
    organization_id: str
    stripe_customer_id: str
    stripe_subscription_id: typing.Optional[str] = None
    trial_ends_at: typing.Optional[dt.datetime] = None
    grace_period_ends_at: typing.Optional[dt.datetime] = None
    payment_method_added: typing.Optional[bool] = None
    current_period_start: typing.Optional[dt.datetime] = None
    current_period_end: typing.Optional[dt.datetime] = None
    cancel_at_period_end: typing.Optional[bool] = None
    pending_plan_change: typing.Optional[BillingPlan] = None
    pending_plan_change_at: typing.Optional[dt.datetime] = None
    payment_method_id: typing.Optional[str] = None
    last_payment_status: typing.Optional[str] = None
    last_payment_at: typing.Optional[dt.datetime] = None
    billing_metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    has_yearly_prepay: typing.Optional[bool] = None
    yearly_prepay_started_at: typing.Optional[dt.datetime] = None
    yearly_prepay_expires_at: typing.Optional[dt.datetime] = None
    yearly_prepay_amount_cents: typing.Optional[int] = None
    yearly_prepay_coupon_id: typing.Optional[str] = None
    yearly_prepay_payment_intent_id: typing.Optional[str] = None
    created_at: dt.datetime
    modified_at: dt.datetime
    current_period: typing.Optional[BillingPeriod] = pydantic.Field(default=None)
    """
    Current active billing period
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ActionCheckRequest,
        ActionCheckResponse,
        AgentApiKey,
        AgentApiKeyClusterManagement,
        AgentApiKeyCollectionStats,
        AgentApiKeyCreate,
        AgentApiKeyRevoke,
        AgentApiKeyStatus,
        AgentApiKeySummary,
        AgentApiKeyUpdate,
        AgentApiKeyWithClusters,
        AgentApiKeyWithFullKey,
        Any,
        ApiKey,
        ApiKeyCreate,
        AuditActionType,
        AuditLog,
        AuditLogCategory,
        AuditLogLevel,
        AuditLogSearchResult,
        AuthProvider,
        AuthProviderAuthentication,
        AuthProviderConnection,
        AuthProviderConnectionCreate,
        AuthProviderConnectionUpdate,
        AuthenticationDetails,
        AuthenticationMethod,
        BillingPeriod,
        BillingPeriodStatus,
        BillingPeriodUsage,
        BillingPlan,
        BillingStatus,
        BillingTransition,
        BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost,
        BulkOperationResult,
        BulkRagUpdate,
        BulkSensitivityUpdate,
        BulkStatusUpdate,
        CatalogAuditLog,
        CatalogAuditLogSummary,
        CatalogCollectionStats,
        CatalogItem,
        CatalogItemGovernanceUpdate,
        CatalogItemSummary,
        CatalogItemWithGovernance,
        CatalogStatus,
        CatalogTreeNode,
        CatalogTreeView,
        CheckoutSessionRequest,
        CheckoutSessionResponse,
        Collection,
        CollectionMember,
        CollectionMemberAdd,
        CollectionRole,
        CollectionStatus,
        CollectionUpdate,
        ComplianceReport,
        ConfigField,
        ConfigValues,
        Connection,
        ConnectionStatus,
        ContentUpdateStats,
        ContextCluster,
        ContextClusterCollectionStats,
        ContextClusterCreate,
        ContextClusterItemManagement,
        ContextClusterStatus,
        ContextClusterSummary,
        ContextClusterUpdate,
        ContextClusterWithItems,
        CustomerPortalRequest,
        CustomerPortalResponse,
        DashboardStats,
        DatetimeRange,
        Destination,
        DestinationWithAuthenticationFields,
        DirectAuthentication,
        EmbeddingModel,
        EmbeddingModelWithAuthenticationFields,
        EntityCount,
        EntityCountWithDefinition,
        EntityDefinition,
        EntityDefinitionCreate,
        EntityDefinitionCreateEntitySchema,
        EntityDefinitionEntitySchema,
        EntitySummary,
        EntityType,
        EntityTypeStats,
        Except,
        FeatureFlag,
        FieldCondition,
        Fields,
        Filter,
        FilterMustItem,
        FilterMustNot,
        FilterMustNotItem,
        FilterShouldItem,
        GeoBoundingBox,
        GeoLineString,
        GeoPoint,
        GeoPolygon,
        GeoRadius,
        GovernanceActivityFeed,
        GraphLink,
        GraphNode,
        GraphResponse,
        Gt,
        Gte,
        HasIdCondition,
        HasIdConditionHasIdItem,
        HasVectorCondition,
        HttpValidationError,
        IntegrationCredentialInDb,
        IntegrationCredentialRawCreate,
        IntegrationType,
        InvitationCreate,
        InvitationResponse,
        IsEmptyCondition,
        IsNullCondition,
        LegacySearchRequest,
        LegacySearchRequestSearchMethod,
        LegacySearchResponse,
        Lt,
        Lte,
        Match,
        MatchAny,
        MatchExcept,
        MatchPhrase,
        MatchText,
        MatchTextAny,
        MatchValue,
        MemberResponse,
        MessageResponse,
        MinShould,
        MinShouldConditionsItem,
        MinuteLevelScheduleConfig,
        Must,
        Nested,
        NestedCondition,
        OAuthBrowserAuthentication,
        OAuthTokenAuthentication,
        OAuthType,
        Organization,
        OrganizationBilling,
        OrganizationCreate,
        OrganizationCreationValidation,
        OrganizationCreationValidationRequest,
        OrganizationMetrics,
        OrganizationReadinessResponse,
        OrganizationWithRole,
        PayloadField,
        QueryExpansionStrategy,
        RagEntityList,
        RagGovernanceReport,
        Range,
        ResponseType,
        RetrievalStrategy,
        S3ConfigRequest,
        S3ConfigResponse,
        ScheduleConfig,
        ScheduleDetails,
        ScheduleResponse,
        SearchQueryResponse,
        SearchRequest,
        SearchResponse,
        SearchStatus,
        SensitivityLevel,
        Should,
        SingleActionCheckResponse,
        SingleActionCheckResponseReason,
        Source,
        SourceConnection,
        SourceConnectionJob,
        SourceConnectionListItem,
        SourceConnectionStatus,
        SourceDistribution,
        SourceRateLimit,
        SourceRateLimitResponse,
        SourceRateLimitUpdateRequest,
        SubscriptionInfo,
        Sync,
        SyncCreate,
        SyncDetails,
        SyncJob,
        SyncJobAuditSummary,
        SyncJobDetails,
        SyncJobStatus,
        SyncStatus,
        SyncUpdate,
        SyncWithSourceConnection,
        Transformer,
        TransformerCreate,
        TransformerUpdate,
        UpdatePlanRequest,
        UsageDashboard,
        UsageSnapshot,
        UsageTrend,
        User,
        UserActivitySummary,
        UserCreate,
        UserOrganization,
        ValidationError,
        ValidationErrorLocItem,
        Value,
        ValuesCount,
    )
    from .errors import UnprocessableEntityError
    from . import collections, source_connections, sources
    from .client import AsyncGraffoSDK, GraffoSDK
    from .collections import (
        SearchCollectionsReadableIdSearchPostRequestBody,
        SearchCollectionsReadableIdSearchPostResponse,
    )
    from .environment import GraffoSDKEnvironment
    from .source_connections import Authentication
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "ActionCheckRequest": ".types",
    "ActionCheckResponse": ".types",
    "AgentApiKey": ".types",
    "AgentApiKeyClusterManagement": ".types",
    "AgentApiKeyCollectionStats": ".types",
    "AgentApiKeyCreate": ".types",
    "AgentApiKeyRevoke": ".types",
    "AgentApiKeyStatus": ".types",
    "AgentApiKeySummary": ".types",
    "AgentApiKeyUpdate": ".types",
    "AgentApiKeyWithClusters": ".types",
    "AgentApiKeyWithFullKey": ".types",
    "Any": ".types",
    "ApiKey": ".types",
    "ApiKeyCreate": ".types",
    "AsyncGraffoSDK": ".client",
    "AuditActionType": ".types",
    "AuditLog": ".types",
    "AuditLogCategory": ".types",
    "AuditLogLevel": ".types",
    "AuditLogSearchResult": ".types",
    "AuthProvider": ".types",
    "AuthProviderAuthentication": ".types",
    "AuthProviderConnection": ".types",
    "AuthProviderConnectionCreate": ".types",
    "AuthProviderConnectionUpdate": ".types",
    "Authentication": ".source_connections",
    "AuthenticationDetails": ".types",
    "AuthenticationMethod": ".types",
    "BillingPeriod": ".types",
    "BillingPeriodStatus": ".types",
    "BillingPeriodUsage": ".types",
    "BillingPlan": ".types",
    "BillingStatus": ".types",
    "BillingTransition": ".types",
    "BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost": ".types",
    "BulkOperationResult": ".types",
    "BulkRagUpdate": ".types",
    "BulkSensitivityUpdate": ".types",
    "BulkStatusUpdate": ".types",
    "CatalogAuditLog": ".types",
    "CatalogAuditLogSummary": ".types",
    "CatalogCollectionStats": ".types",
    "CatalogItem": ".types",
    "CatalogItemGovernanceUpdate": ".types",
    "CatalogItemSummary": ".types",
    "CatalogItemWithGovernance": ".types",
    "CatalogStatus": ".types",
    "CatalogTreeNode": ".types",
    "CatalogTreeView": ".types",
    "CheckoutSessionRequest": ".types",
    "CheckoutSessionResponse": ".types",
    "Collection": ".types",
    "CollectionMember": ".types",
    "CollectionMemberAdd": ".types",
    "CollectionRole": ".types",
    "CollectionStatus": ".types",
    "CollectionUpdate": ".types",
    "ComplianceReport": ".types",
    "ConfigField": ".types",
    "ConfigValues": ".types",
    "Connection": ".types",
    "ConnectionStatus": ".types",
    "ContentUpdateStats": ".types",
    "ContextCluster": ".types",
    "ContextClusterCollectionStats": ".types",
    "ContextClusterCreate": ".types",
    "ContextClusterItemManagement": ".types",
    "ContextClusterStatus": ".types",
    "ContextClusterSummary": ".types",
    "ContextClusterUpdate": ".types",
    "ContextClusterWithItems": ".types",
    "CustomerPortalRequest": ".types",
    "CustomerPortalResponse": ".types",
    "DashboardStats": ".types",
    "DatetimeRange": ".types",
    "Destination": ".types",
    "DestinationWithAuthenticationFields": ".types",
    "DirectAuthentication": ".types",
    "EmbeddingModel": ".types",
    "EmbeddingModelWithAuthenticationFields": ".types",
    "EntityCount": ".types",
    "EntityCountWithDefinition": ".types",
    "EntityDefinition": ".types",
    "EntityDefinitionCreate": ".types",
    "EntityDefinitionCreateEntitySchema": ".types",
    "EntityDefinitionEntitySchema": ".types",
    "EntitySummary": ".types",
    "EntityType": ".types",
    "EntityTypeStats": ".types",
    "Except": ".types",
    "FeatureFlag": ".types",
    "FieldCondition": ".types",
    "Fields": ".types",
    "Filter": ".types",
    "FilterMustItem": ".types",
    "FilterMustNot": ".types",
    "FilterMustNotItem": ".types",
    "FilterShouldItem": ".types",
    "GeoBoundingBox": ".types",
    "GeoLineString": ".types",
    "GeoPoint": ".types",
    "GeoPolygon": ".types",
    "GeoRadius": ".types",
    "GovernanceActivityFeed": ".types",
    "GraffoSDK": ".client",
    "GraffoSDKEnvironment": ".environment",
    "GraphLink": ".types",
    "GraphNode": ".types",
    "GraphResponse": ".types",
    "Gt": ".types",
    "Gte": ".types",
    "HasIdCondition": ".types",
    "HasIdConditionHasIdItem": ".types",
    "HasVectorCondition": ".types",
    "HttpValidationError": ".types",
    "IntegrationCredentialInDb": ".types",
    "IntegrationCredentialRawCreate": ".types",
    "IntegrationType": ".types",
    "InvitationCreate": ".types",
    "InvitationResponse": ".types",
    "IsEmptyCondition": ".types",
    "IsNullCondition": ".types",
    "LegacySearchRequest": ".types",
    "LegacySearchRequestSearchMethod": ".types",
    "LegacySearchResponse": ".types",
    "Lt": ".types",
    "Lte": ".types",
    "Match": ".types",
    "MatchAny": ".types",
    "MatchExcept": ".types",
    "MatchPhrase": ".types",
    "MatchText": ".types",
    "MatchTextAny": ".types",
    "MatchValue": ".types",
    "MemberResponse": ".types",
    "MessageResponse": ".types",
    "MinShould": ".types",
    "MinShouldConditionsItem": ".types",
    "MinuteLevelScheduleConfig": ".types",
    "Must": ".types",
    "Nested": ".types",
    "NestedCondition": ".types",
    "OAuthBrowserAuthentication": ".types",
    "OAuthTokenAuthentication": ".types",
    "OAuthType": ".types",
    "Organization": ".types",
    "OrganizationBilling": ".types",
    "OrganizationCreate": ".types",
    "OrganizationCreationValidation": ".types",
    "OrganizationCreationValidationRequest": ".types",
    "OrganizationMetrics": ".types",
    "OrganizationReadinessResponse": ".types",
    "OrganizationWithRole": ".types",
    "PayloadField": ".types",
    "QueryExpansionStrategy": ".types",
    "RagEntityList": ".types",
    "RagGovernanceReport": ".types",
    "Range": ".types",
    "ResponseType": ".types",
    "RetrievalStrategy": ".types",
    "S3ConfigRequest": ".types",
    "S3ConfigResponse": ".types",
    "ScheduleConfig": ".types",
    "ScheduleDetails": ".types",
    "ScheduleResponse": ".types",
    "SearchCollectionsReadableIdSearchPostRequestBody": ".collections",
    "SearchCollectionsReadableIdSearchPostResponse": ".collections",
    "SearchQueryResponse": ".types",
    "SearchRequest": ".types",
    "SearchResponse": ".types",
    "SearchStatus": ".types",
    "SensitivityLevel": ".types",
    "Should": ".types",
    "SingleActionCheckResponse": ".types",
    "SingleActionCheckResponseReason": ".types",
    "Source": ".types",
    "SourceConnection": ".types",
    "SourceConnectionJob": ".types",
    "SourceConnectionListItem": ".types",
    "SourceConnectionStatus": ".types",
    "SourceDistribution": ".types",
    "SourceRateLimit": ".types",
    "SourceRateLimitResponse": ".types",
    "SourceRateLimitUpdateRequest": ".types",
    "SubscriptionInfo": ".types",
    "Sync": ".types",
    "SyncCreate": ".types",
    "SyncDetails": ".types",
    "SyncJob": ".types",
    "SyncJobAuditSummary": ".types",
    "SyncJobDetails": ".types",
    "SyncJobStatus": ".types",
    "SyncStatus": ".types",
    "SyncUpdate": ".types",
    "SyncWithSourceConnection": ".types",
    "Transformer": ".types",
    "TransformerCreate": ".types",
    "TransformerUpdate": ".types",
    "UnprocessableEntityError": ".errors",
    "UpdatePlanRequest": ".types",
    "UsageDashboard": ".types",
    "UsageSnapshot": ".types",
    "UsageTrend": ".types",
    "User": ".types",
    "UserActivitySummary": ".types",
    "UserCreate": ".types",
    "UserOrganization": ".types",
    "ValidationError": ".types",
    "ValidationErrorLocItem": ".types",
    "Value": ".types",
    "ValuesCount": ".types",
    "__version__": ".version",
    "collections": ".collections",
    "source_connections": ".source_connections",
    "sources": ".sources",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ActionCheckRequest",
    "ActionCheckResponse",
    "AgentApiKey",
    "AgentApiKeyClusterManagement",
    "AgentApiKeyCollectionStats",
    "AgentApiKeyCreate",
    "AgentApiKeyRevoke",
    "AgentApiKeyStatus",
    "AgentApiKeySummary",
    "AgentApiKeyUpdate",
    "AgentApiKeyWithClusters",
    "AgentApiKeyWithFullKey",
    "Any",
    "ApiKey",
    "ApiKeyCreate",
    "AsyncGraffoSDK",
    "AuditActionType",
    "AuditLog",
    "AuditLogCategory",
    "AuditLogLevel",
    "AuditLogSearchResult",
    "AuthProvider",
    "AuthProviderAuthentication",
    "AuthProviderConnection",
    "AuthProviderConnectionCreate",
    "AuthProviderConnectionUpdate",
    "Authentication",
    "AuthenticationDetails",
    "AuthenticationMethod",
    "BillingPeriod",
    "BillingPeriodStatus",
    "BillingPeriodUsage",
    "BillingPlan",
    "BillingStatus",
    "BillingTransition",
    "BodyConnectSlackWithTokenConnectionsDirectTokenSlackPost",
    "BulkOperationResult",
    "BulkRagUpdate",
    "BulkSensitivityUpdate",
    "BulkStatusUpdate",
    "CatalogAuditLog",
    "CatalogAuditLogSummary",
    "CatalogCollectionStats",
    "CatalogItem",
    "CatalogItemGovernanceUpdate",
    "CatalogItemSummary",
    "CatalogItemWithGovernance",
    "CatalogStatus",
    "CatalogTreeNode",
    "CatalogTreeView",
    "CheckoutSessionRequest",
    "CheckoutSessionResponse",
    "Collection",
    "CollectionMember",
    "CollectionMemberAdd",
    "CollectionRole",
    "CollectionStatus",
    "CollectionUpdate",
    "ComplianceReport",
    "ConfigField",
    "ConfigValues",
    "Connection",
    "ConnectionStatus",
    "ContentUpdateStats",
    "ContextCluster",
    "ContextClusterCollectionStats",
    "ContextClusterCreate",
    "ContextClusterItemManagement",
    "ContextClusterStatus",
    "ContextClusterSummary",
    "ContextClusterUpdate",
    "ContextClusterWithItems",
    "CustomerPortalRequest",
    "CustomerPortalResponse",
    "DashboardStats",
    "DatetimeRange",
    "Destination",
    "DestinationWithAuthenticationFields",
    "DirectAuthentication",
    "EmbeddingModel",
    "EmbeddingModelWithAuthenticationFields",
    "EntityCount",
    "EntityCountWithDefinition",
    "EntityDefinition",
    "EntityDefinitionCreate",
    "EntityDefinitionCreateEntitySchema",
    "EntityDefinitionEntitySchema",
    "EntitySummary",
    "EntityType",
    "EntityTypeStats",
    "Except",
    "FeatureFlag",
    "FieldCondition",
    "Fields",
    "Filter",
    "FilterMustItem",
    "FilterMustNot",
    "FilterMustNotItem",
    "FilterShouldItem",
    "GeoBoundingBox",
    "GeoLineString",
    "GeoPoint",
    "GeoPolygon",
    "GeoRadius",
    "GovernanceActivityFeed",
    "GraffoSDK",
    "GraffoSDKEnvironment",
    "GraphLink",
    "GraphNode",
    "GraphResponse",
    "Gt",
    "Gte",
    "HasIdCondition",
    "HasIdConditionHasIdItem",
    "HasVectorCondition",
    "HttpValidationError",
    "IntegrationCredentialInDb",
    "IntegrationCredentialRawCreate",
    "IntegrationType",
    "InvitationCreate",
    "InvitationResponse",
    "IsEmptyCondition",
    "IsNullCondition",
    "LegacySearchRequest",
    "LegacySearchRequestSearchMethod",
    "LegacySearchResponse",
    "Lt",
    "Lte",
    "Match",
    "MatchAny",
    "MatchExcept",
    "MatchPhrase",
    "MatchText",
    "MatchTextAny",
    "MatchValue",
    "MemberResponse",
    "MessageResponse",
    "MinShould",
    "MinShouldConditionsItem",
    "MinuteLevelScheduleConfig",
    "Must",
    "Nested",
    "NestedCondition",
    "OAuthBrowserAuthentication",
    "OAuthTokenAuthentication",
    "OAuthType",
    "Organization",
    "OrganizationBilling",
    "OrganizationCreate",
    "OrganizationCreationValidation",
    "OrganizationCreationValidationRequest",
    "OrganizationMetrics",
    "OrganizationReadinessResponse",
    "OrganizationWithRole",
    "PayloadField",
    "QueryExpansionStrategy",
    "RagEntityList",
    "RagGovernanceReport",
    "Range",
    "ResponseType",
    "RetrievalStrategy",
    "S3ConfigRequest",
    "S3ConfigResponse",
    "ScheduleConfig",
    "ScheduleDetails",
    "ScheduleResponse",
    "SearchCollectionsReadableIdSearchPostRequestBody",
    "SearchCollectionsReadableIdSearchPostResponse",
    "SearchQueryResponse",
    "SearchRequest",
    "SearchResponse",
    "SearchStatus",
    "SensitivityLevel",
    "Should",
    "SingleActionCheckResponse",
    "SingleActionCheckResponseReason",
    "Source",
    "SourceConnection",
    "SourceConnectionJob",
    "SourceConnectionListItem",
    "SourceConnectionStatus",
    "SourceDistribution",
    "SourceRateLimit",
    "SourceRateLimitResponse",
    "SourceRateLimitUpdateRequest",
    "SubscriptionInfo",
    "Sync",
    "SyncCreate",
    "SyncDetails",
    "SyncJob",
    "SyncJobAuditSummary",
    "SyncJobDetails",
    "SyncJobStatus",
    "SyncStatus",
    "SyncUpdate",
    "SyncWithSourceConnection",
    "Transformer",
    "TransformerCreate",
    "TransformerUpdate",
    "UnprocessableEntityError",
    "UpdatePlanRequest",
    "UsageDashboard",
    "UsageSnapshot",
    "UsageTrend",
    "User",
    "UserActivitySummary",
    "UserCreate",
    "UserOrganization",
    "ValidationError",
    "ValidationErrorLocItem",
    "Value",
    "ValuesCount",
    "__version__",
    "collections",
    "source_connections",
    "sources",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .fields import Fields


class Source(UniversalBaseModel):
    """
    Complete source representation with authentication and configuration schemas.
    """

    name: str = pydantic.Field()
    """
    Human-readable name of the data source connector (e.g., 'GitHub', 'Stripe', 'PostgreSQL').
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description explaining what data this source can extract and its typical use cases.
    """

    auth_methods: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of supported authentication methods (e.g., 'direct', 'oauth_browser').
    """

    oauth_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth token type for OAuth sources (e.g., 'access_only', 'with_refresh').
    """

    requires_byoc: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this OAuth source requires users to bring their own client.
    """

    auth_config_class: typing.Optional[str] = pydantic.Field(default=None)
    """
    Python class name that defines the authentication configuration fields required for this source (only for DIRECT auth).
    """

    config_class: typing.Optional[str] = pydantic.Field(default=None)
    """
    Python class name that defines the source-specific configuration options and parameters.
    """

    short_name: str = pydantic.Field()
    """
    Technical identifier used internally to reference this source type. Must be unique across all sources.
    """

    class_name: str = pydantic.Field()
    """
    Python class name of the source implementation that handles data extraction logic.
    """

    output_entity_definition_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of entity definition IDs that this source can produce. Defines the data schema and structure that this connector outputs.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Categorization tags to help users discover and filter sources by domain or use case.
    """

    supports_continuous: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this source supports cursor-based continuous syncing for incremental data extraction. Sources with this capability can track their sync position and resume from where they left off.
    """

    federated_search: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this source uses federated search instead of traditional syncing. Federated search sources query data in real-time during searches rather than syncing and indexing all data beforehand.
    """

    supports_temporal_relevance: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this source's entities have timestamps that enable recency-based ranking. Sources without file-level timestamps (e.g., code repositories) cannot use temporal relevance for search result weighting.
    """

    rate_limit_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    Rate limiting level for this source: 'org' (organization-wide), 'connection' (per-connection/per-user), or None (no rate limiting).
    """

    id: str = pydantic.Field()
    """
    Unique system identifier for this source type. Generated automatically when the source is registered.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp when this source type was registered in the system (ISO 8601 format).
    """

    modified_at: dt.datetime = pydantic.Field()
    """
    Timestamp when this source type was last updated (ISO 8601 format).
    """

    auth_fields: typing.Optional[Fields] = pydantic.Field(default=None)
    """
    Schema definition for authentication fields required to connect to this source. Only present for sources using DIRECT authentication. OAuth sources handle authentication through browser flows.
    """

    config_fields: Fields = pydantic.Field()
    """
    Schema definition for configuration fields required to customize this source. Describes field types, validation rules, and user interface hints.
    """

    supported_auth_providers: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of auth provider short names that support this source (e.g., ['composio', 'pipedream']). Computed dynamically for API responses. This field is not stored in the database.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .catalog_status import CatalogStatus
from .sensitivity_level import SensitivityLevel


class CatalogItemWithGovernance(UniversalBaseModel):
    """
    Catalog item with governance-specific metadata.
    """

    entity_id: str
    title: str
    description: typing.Optional[str] = None
    tags: typing.Optional[typing.List[str]] = None
    asset_type: str
    source_external_id: typing.Optional[str] = None
    source_url: typing.Optional[str] = None
    source_name: typing.Optional[str] = None
    parent_catalog_id: typing.Optional[str] = None
    hierarchy_path: typing.Optional[str] = None
    depth_level: typing.Optional[int] = None
    include_in_rag: typing.Optional[bool] = None
    status: typing.Optional[CatalogStatus] = None
    sensitivity_level: typing.Optional[SensitivityLevel] = None
    id: str
    sync_id: str
    collection_id: str
    entity_db_id: typing.Optional[str] = None
    entity_definition_id: typing.Optional[str] = None
    source_id: typing.Optional[str] = None
    content_hash: typing.Optional[str] = None
    last_content_update: typing.Optional[str] = None
    last_embedded: typing.Optional[str] = None
    chunk_count: typing.Optional[int] = None
    created_at: typing.Optional[str] = None
    modified_at: typing.Optional[str] = None
    created_by: typing.Optional[str] = None
    updated_by: typing.Optional[str] = None
    rag_eligibility: typing.Dict[str, typing.Optional[typing.Any]]
    governance_health: typing.Dict[str, typing.Optional[typing.Any]]
    last_audit_entry: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_status import ConnectionStatus
from .integration_type import IntegrationType


class Connection(UniversalBaseModel):
    """
    Schema for connection with config fields.
    """

    name: str = pydantic.Field()
    """
    Human-readable display name for the connection.
    """

    readable_id: str = pydantic.Field()
    """
    URL-safe unique identifier used in API endpoints. This becomes non-optional once the connection is created.
    """

    description: typing.Optional[str] = None
    integration_type: IntegrationType
    integration_credential_id: typing.Optional[str] = None
    status: ConnectionStatus
    short_name: str
    id: str = pydantic.Field()
    """
    Unique system identifier for the connection.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the connection was created (ISO 8601 format).
    """

    modified_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the connection was last modified (ISO 8601 format).
    """

    organization_id: typing.Optional[str] = None
    created_by_email: typing.Optional[str] = None
    modified_by_email: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .feature_flag import FeatureFlag
from .organization_billing import OrganizationBilling


class Organization(UniversalBaseModel):
    """
    Organization schema with billing and feature information.

    This is the primary organization schema used in API contexts, enriched with
    billing (including current period) and feature flags for efficient access
    without additional database queries.

    Access billing info compositionally:
    - organization.billing.plan
    - organization.billing.status
    - organization.billing.current_period
    """

    name: str
    description: typing.Optional[str] = None
    auth_0_org_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="auth0_org_id")] = None
    org_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization type (personal or manual)
    """

    id: str
    created_at: dt.datetime
    modified_at: dt.datetime
    org_metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    enabled_features: typing.Optional[typing.List[FeatureFlag]] = pydantic.Field(default=None)
    """
    List of enabled feature flags for this organization
    """

    billing: typing.Optional[OrganizationBilling] = pydantic.Field(default=None)
    """
    Complete billing information including current period
    """

    checkout_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Stripe checkout URL for completing paid plan setup, when applicable
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .fields import Fields


class AuthProvider(UniversalBaseModel):
    """
    Schema for auth provider response.
    """

    name: str = pydantic.Field()
    """
    Human-readable name of the auth provider (e.g., 'Google OAuth', 'GitHub')
    """

    short_name: str = pydantic.Field()
    """
    Technical identifier used internally to reference this auth provider. Must be unique.
    """

    class_name: str = pydantic.Field()
    """
    Python class name of the auth provider implementation
    """

    auth_config_class: str = pydantic.Field()
    """
    Python class name that defines the authentication configuration fields
    """

    config_class: str = pydantic.Field()
    """
    Python class name that defines the auth provider-specific configuration
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description explaining what this auth provider offers
    """

    organization_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization identifier for custom auth providers. System providers have this set to null.
    """

    id: str
    created_at: dt.datetime
    modified_at: dt.datetime
    auth_fields: typing.Optional[Fields] = pydantic.Field(default=None)
    """
    Dynamically populated field definitions for authentication configuration. These describe what credentials are required to connect to this auth provider.
    """

    config_fields: typing.Optional[Fields] = pydantic.Field(default=None)
    """
    Dynamically populated field definitions for auth provider-specific configuration. These describe what additional configuration is required when using this auth provider to connect to a source (e.g., auth_config_id and account_id for Composio).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .geo_line_string import GeoLineString


class GeoPolygon(UniversalBaseModel):
    """
    Geo filter request  Matches coordinates inside the polygon, defined by `exterior` and `interiors`
    """

    exterior: GeoLineString = pydantic.Field()
    """
    Geo filter request  Matches coordinates inside the polygon, defined by `exterior` and `interiors`
    """

    interiors: typing.Optional[typing.List[GeoLineString]] = pydantic.Field(default=None)
    """
    Interior lines (if present) bound holes within the surface each GeoLineString must consist of a minimum of 4 points, and the first and last points must be the same.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
